from cmd2 import CommandResult
from cmd2_ext_test import ExternalTestMixin
from dev_shell.tests.fixtures import CmdAppBaseTestCase
from django import __version__

from djfritz_project.dev_shell import DevShellApp, get_devshell_app_kwargs


class DevShellAppTester(ExternalTestMixin, DevShellApp):
    pass


class DevShellAppBaseTestCase(CmdAppBaseTestCase):
    """
    Base class for dev-shell tests
    """

    def get_app_instance(self):
        # Init the test app with the same kwargs as the real app
        # see: dev_shell.cmd2app.devshell_cmdloop()
        app = DevShellAppTester(**get_devshell_app_kwargs())
        return app


class DjFritzDevShellTestCase(DevShellAppBaseTestCase):
    def test_help_raw(self):
        out = self.app.app_cmd('help')

        assert isinstance(out, CommandResult)
        assert 'Documented commands' in out.stdout

        assert 'Documented commands' in out.stdout

    def test_help_via_execute(self):
        stdout, stderr = self.execute('help')
        assert stderr == ''
        assert 'Documented commands' in stdout

    def test_manage(self):
        stdout, stderr = self.execute('manage --version')
        assert stderr == ''
        assert 'manage.py --version' in stdout
        assert __version__ in stdout



import os
import shutil
import subprocess
from pathlib import Path

from django.conf import settings
from django.core import checks
from django.test import TestCase

import djfritz


PACKAGE_ROOT = Path(djfritz.__file__).parent.parent


def assert_file_contains_string(file_path, string):
    with file_path.open('r') as f:
        for line in f:
            if string in line:
                return
    raise AssertionError(f'File {file_path} does not contain {string!r} !')


def test_version(package_root=None, version=None):
    if package_root is None:
        package_root = PACKAGE_ROOT

    if version is None:
        version = djfritz.__version__

    if 'dev' not in version and 'rc' not in version:
        version_string = f'v{version}'

        assert_file_contains_string(
            file_path=Path(package_root, 'README.md'), string=version_string
        )

    assert_file_contains_string(
        file_path=Path(package_root, 'pyproject.toml'), string=f'version = "{version}"'
    )


def test_poetry_check(package_root=None):
    if package_root is None:
        package_root = PACKAGE_ROOT

    poerty_bin = shutil.which('poetry')

    output = subprocess.check_output(
        [poerty_bin, 'check'],
        universal_newlines=True,
        env=os.environ,
        stderr=subprocess.STDOUT,
        cwd=str(package_root),
    )
    print(output)
    assert output == 'All set!\n'


class ProjectSettingsTestCase(TestCase):
    def test_base_path(self):
        base_path = settings.BASE_PATH
        assert base_path.is_dir()
        assert Path(base_path, 'djfritz').is_dir()
        assert Path(base_path, 'djfritz_project').is_dir()

    def test_template_dirs(self):
        assert len(settings.TEMPLATES) == 1
        dirs = settings.TEMPLATES[0].get('DIRS')
        assert len(dirs) == 1
        template_path = Path(dirs[0]).resolve()
        assert template_path.is_dir()

    def test_manage_check(self):
        all_issues = checks.run_checks(
            app_configs=None,
            tags=None,
            include_deployment_checks=True,
            databases=None,
        )
        if all_issues:
            print('=' * 100)
            for issue in all_issues:
                print(issue)
            print('=' * 100)
            raise AssertionError('There are check issues!')



from bx_django_utils.view_utils.dynamic_menu_urls import DynamicViewMenu

from djfritz.views.diagnose import FritzBoxConnectionView, ListBoxServicesView
from djfritz.views.host_infomation import HostInformationView, MeshTopologyView


dynamic_view_menu = DynamicViewMenu()


dynamic_view_menu.add_views(
    app_name='djfritz',
    menu=(
        (
            'Host information',
            {
                'views': (
                    (HostInformationView, 'host_information'),
                    (MeshTopologyView, 'mest_information'),
                )
            },
        ),
        (
            'Diagnose',
            {
                'views': (
                    (FritzBoxConnectionView, 'diagnose_connection'),
                    (ListBoxServicesView, 'list_box_services'),
                )
            },
        ),
    ),
)



from django.core.management.base import BaseCommand

from djfritz.services.hosts import update_hosts


class Command(BaseCommand):
    help = 'Update the Host model from current FritzBox Connection'

    def handle(self, *args, **options):
        message = update_hosts()
        print(message)



from pathlib import Path

from poetry_publish.publish import poetry_publish
from poetry_publish.utils.subprocess_utils import verbose_check_call

# https://github.com/jedie/django-fritzconnection
import djfritz


PACKAGE_ROOT = Path(djfritz.__file__).parent.parent.parent


def publish():
    """
    Publish to PyPi
    Call this via:
        $ poetry run publish
    """
    verbose_check_call('make', 'pytest')  # don't publish if tests fail
    verbose_check_call('make', 'fix-code-style')  # don't publish if code style wrong

    poetry_publish(
        package_root=PACKAGE_ROOT,
        version=djfritz.__version__,
        creole_readme=True,  # don't publish if README.rst is not up-to-date
    )



from django import template
from django.template.loader import render_to_string

from djfritz.fritz_connection import get_fritz_connection
from djfritz.views import dynamic_view_menu


register = template.Library()


@register.simple_tag(takes_context=True)
def view_menu(context):
    context = {
        'menu': dynamic_view_menu.menu,
    }
    return render_to_string('djfritz/includes/dynamic_view_menu.html', context)


@register.simple_tag(takes_context=True)
def fritzbox_link(context):
    fc = get_fritz_connection()
    if not fc:
        return '<not connected>'

    context = {
        'modelname': fc.modelname,
        'address': fc.soaper.address,
        'system_version': fc.system_version,
    }
    return render_to_string('djfritz/includes/fritzbox_link.html', context)



from django.contrib.admin import site
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.decorators import method_decorator


class DjangoAdminContextMixin:
    def get_context_data(self, **context):
        extra = site.each_context(self.request)
        context.update(**extra)
        return super().get_context_data(**context)


class OnlyStaffUserMixin:
    title = None

    @method_decorator(staff_member_required)
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **context):
        assert self.title is not None
        context['title'] = self.title
        return super().get_context_data(**context)



"""
    created 24.03.2022 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2022 by the django-fritzconnection team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

__version__ = "0.0.2.rc0"



import logging
from uuid import UUID

from bx_django_utils.models.manipulate import CreateOrUpdateResult
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import path, reverse
from fritzconnection.core.exceptions import FritzConnectionException
from reversion_compare.admin import CompareVersionAdmin

from djfritz.models.hosts import HostModel
from djfritz.services.hosts import set_wan_access_state, update_host, update_hosts


logger = logging.getLogger(__name__)


@admin.register(HostModel)
class HostModelAdmin(CompareVersionAdmin):
    change_list_template = 'admin/djfritz/hostmodel/change_list.html'

    search_fields = ('name', 'tags__name')
    list_display = (
        'mac',
        'ip_v4',
        'name',
        'last_status',
        'create_dt',
        'update_dt',
        'wan_access',
    )
    list_display_links = ('name',)
    list_filter = ('last_status', 'wan_access', 'interface_type', 'address_source', 'tags')
    date_hierarchy = 'create_dt'
    readonly_fields = ('wan_access',)

    def get_urls(self):
        urls = super().get_urls()
        opts = self.model._meta
        urls += [
            path(
                route='update_hosts',
                view=self.admin_site.admin_view(self.update_hosts_view, cacheable=False),
                name=f'{opts.app_label}_{opts.model_name}_update_hosts',
            ),
            path(
                route='update_host/<uuid:object_id>',
                view=self.admin_site.admin_view(self.update_host_view, cacheable=False),
                name=f'{opts.app_label}_{opts.model_name}_update_host',
            ),
            path(
                route='allow_wan_access/<uuid:object_id>',
                view=self.admin_site.admin_view(self.allow_wan_access_view, cacheable=False),
                name=f'{opts.app_label}_{opts.model_name}_allow_wan_access',
            ),
            path(
                route='disallow_wan_access/<uuid:object_id>',
                view=self.admin_site.admin_view(self.disallow_wan_access_view, cacheable=False),
                name=f'{opts.app_label}_{opts.model_name}_disallow_wan_access',
            )
        ]
        return urls

    def get_object_or_404(self, object_id):
        assert isinstance(object_id, UUID)
        obj = get_object_or_404(self.model, pk=object_id)
        return obj

    def redirect2change(self, obj):
        url = obj.get_change_url()
        return HttpResponseRedirect(url)

    def update_hosts_view(self, request, extra_context=None):
        msg = update_hosts()
        messages.info(request, msg)

        opts = self.model._meta
        url = reverse(
            f'admin:{opts.app_label}_{opts.model_name}_changelist',
            current_app=self.admin_site.name,
        )
        return HttpResponseRedirect(url)

    def update_host_view(self, request, object_id, extra_context=None):
        obj = self.get_object_or_404(object_id)
        result: CreateOrUpdateResult = update_host(host=obj)
        if result.updated_fields:
            messages.success(request, f'Updated: {", ".join(result.updated_fields)}')
        else:
            messages.info(request, 'No changed, all values up-to-date')
        return self.redirect2change(obj)

    def set_wan_access_state(self, request, object_id, allow: bool):
        obj = self.get_object_or_404(object_id)
        try:
            result: CreateOrUpdateResult = set_wan_access_state(host=obj, allow=allow)
        except FritzConnectionException as err:
            msg = f'Error set a new WAN access state: {err}'
            logger.exception(msg)
            messages.error(request, msg)
        else:
            if result.updated_fields:
                obj.refresh_from_db()
                messages.success(request, f'WAN access state changed to: {obj.wan_access}')
            else:
                messages.info(request, 'WAN access state unchanged.')

        return self.redirect2change(obj)

    def allow_wan_access_view(self, request, object_id, extra_context=None):
        logger.info('Allow WAN access to %r', object_id)
        return self.set_wan_access_state(request, object_id, allow=True)

    def disallow_wan_access_view(self, request, object_id, extra_context=None):
        logger.info('Disallow WAN access to %r', object_id)
        return self.set_wan_access_state(request, object_id, allow=False)



from pathlib import Path

from bx_py_utils.test_utils.snapshot import assert_html_snapshot
from django.http import HttpResponse


def assert_html_response_snapshot(
    response: HttpResponse,
    status_code: int = 200,
    validate: bool = True,
):
    """
    TODO: Move to bx_django_utils
    """
    data = response.content.decode('utf-8')

    assert response.status_code == status_code, (
        f'Status code is {response.status_code} but excepted {status_code}'
    )

    assert_html_snapshot(
        got=data,
        self_file_path=Path(__file__),
        validate=validate
    )



import io

from django.core import management
from django.core.management.commands import makemigrations
from django.test import TestCase, override_settings


class TestMigrations(TestCase):
    databases = ['default', ]

    @override_settings(MIGRATION_MODULES={})
    def test_missing_migrations(self):
        output = io.StringIO()
        try:
            management.call_command(
                makemigrations.Command(),
                dry_run=True,
                check_changes=True,
                verbosity=1,
                stdout=output
            )
        except SystemExit as err:
            if err.code != 0:
                raise AssertionError(output.getvalue())



import logging
from pathlib import Path

from debug_toolbar.middleware import show_toolbar
from django.conf import settings


logger = logging.getLogger(__name__)


def djdt_show(request):
    """
    Determining whether the Django Debug Toolbar should show or not.
    """
    if not settings.DEBUG:
        return False

    if Path('/.dockerenv').exists():
        # We run in a docker container
        # skip the `request.META['REMOTE_ADDR'] in settings.INTERNAL_IPS` check.
        return True

    return show_toolbar(request)



# Generated by Django 4.0.3 on 2022-04-01 15:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djfritz', '0001_add_host_model'),
    ]

    operations = [
        migrations.AddField(
            model_name='hostmodel',
            name='ip_v4',
            field=models.GenericIPAddressField(blank=True, help_text='HostModel.ip_v4.help_text', null=True, protocol='IPv4', verbose_name='HostModel.ip_v4.verbose_name'),
        ),
        migrations.AddField(
            model_name='hostmodel',
            name='ip_v6',
            field=models.GenericIPAddressField(blank=True, help_text='HostModel.ip_v6.help_text', null=True, protocol='IPv6', verbose_name='HostModel.ip_v6.verbose_name'),
        ),
    ]



# Generated by Django 4.0.3 on 2022-04-01 16:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djfritz', '0002_hostmodel_ip_v4_hostmodel_ip_v6'),
    ]

    operations = [
        migrations.AddField(
            model_name='hostmodel',
            name='wan_access',
            field=models.CharField(choices=[('granted', 'granted'), ('denied', 'denied'), ('error', 'error'), ('unknown', 'unknown')], default='unknown', editable=False, help_text='HostModel.wan_access.help_text', max_length=7, verbose_name='HostModel.wan_access.verbose_name'),
        ),
    ]



import logging
import time

import reversion
from bx_django_utils.humanize.time import human_timedelta
from bx_django_utils.models.manipulate import CreateOrUpdateResult, create_or_update2
from django.utils.translation import gettext as _
from fritzconnection.lib.fritzhosts import FritzHosts

from djfritz.fritz_connection import FritzHostFilter, get_fritz_connection
from djfritz.models import HostModel


logger = logging.getLogger(__name__)


def update_host(host: HostModel) -> CreateOrUpdateResult:
    mac_address: str = host.mac

    fc = get_fritz_connection()
    fh = FritzHosts(fc=fc)
    data = fh.get_specific_host_entry(mac_address=mac_address)

    ip_v4 = data['NewIPAddress']

    fhf = FritzHostFilter(fc=fc)
    wan_access = fhf.get_wan_access_state(ip=ip_v4)

    with reversion.create_revision():
        result: CreateOrUpdateResult = create_or_update2(
            ModelClass=HostModel,
            lookup={'mac': mac_address},
            ip_v4=ip_v4,
            name=data['NewHostName'],
            last_status=data['NewActive'],
            interface_type=data['NewInterfaceType'] or None,
            address_source=data['NewAddressSource'],
            lease_time_remaining=data['NewLeaseTimeRemaining'],
            wan_access=wan_access,
        )
        if result.created:
            comment = 'Created'
        elif result.updated_fields:
            comment = f'Updated: {", ".join(result.updated_fields)}'
        else:
            comment = 'unchanged'

        logger.info('%s - %s', comment, result.instance)
        reversion.set_comment(comment)
    return result


def update_hosts():
    logger.info('start updating hosts...')
    start_time = time.monotonic()
    fc = get_fritz_connection()
    fh = FritzHosts(fc=fc)
    fhf = FritzHostFilter(fc=fc)
    hosts = fh.get_hosts_info()

    created = 0
    updated = 0
    unchanged = 0
    for host in hosts:

        ip_v4 = host['ip']
        wan_access = fhf.get_wan_access_state(ip=ip_v4)
        logger.info('WAN access state for %r is: %r', ip_v4, wan_access)

        with reversion.create_revision():
            result: CreateOrUpdateResult = create_or_update2(
                ModelClass=HostModel,
                lookup={'mac': host['mac']},
                ip_v4=ip_v4,
                name=host['name'],
                last_status=host['status'],
                interface_type=host['interface_type'] or None,
                address_source=host['address_source'],
                lease_time_remaining=host['lease_time_remaining'],
                wan_access=wan_access,
            )
            if result.created:
                created += 1
                comment = 'Created'
            elif result.updated_fields:
                updated += 1
                comment = f'Updated: {", ".join(result.updated_fields)}'
            else:
                unchanged += 1
                comment = 'unchanged'

            logger.info('%s - %s', comment, result.instance)
            reversion.set_comment(comment)

    duration_str = human_timedelta(time.monotonic() - start_time)
    message = _(
        f'{created} hosts created, {updated} updated and {unchanged} not changed in {duration_str}'
    )
    return message


def set_wan_access_state(host: HostModel, allow: bool) -> CreateOrUpdateResult:
    """
    Set WAN access state for given "host" instance.
    """
    ip_v4: str = host.ip_v4
    if not ip_v4:
        raise RuntimeError('Host IP is unknown! Cannot change the WAN access state without IP!')

    fc = get_fritz_connection()
    fhf = FritzHostFilter(fc=fc)
    if allow:
        state = fhf.allow_wan_access(ip=ip_v4)
    else:
        state = fhf.disallow_wan_access(ip=ip_v4)

    result: CreateOrUpdateResult = create_or_update2(
        ModelClass=HostModel,
        lookup={'mac': host.mac},
        wan_access=state,
    )
    return result



from django.contrib import messages
from django.views.generic import TemplateView
from fritzconnection import __version__ as fc_version

from djfritz.fritz_connection import get_fritz_connection
from djfritz.views.base_views import DjangoAdminContextMixin, OnlyStaffUserMixin


class FritzBoxConnectionView(OnlyStaffUserMixin, DjangoAdminContextMixin, TemplateView):
    title = 'Test FritzBox connection'
    template_name = 'djfritz/diagnose_connection.html'

    def get_context_data(self, **context):
        fc = get_fritz_connection()

        system_info = fc.device_manager.system_info
        if system_info:
            context.update(
                dict(
                    system=system_info[-1],
                    build=system_info[-2],
                    hw_code=system_info[0],
                )
            )

        context.update(
            dict(
                fritzconnection_version=fc_version,
                last_connection=get_fritz_connection.last_connection,
                modelname=fc.modelname,
                address=fc.soaper.address,
                system_version=fc.system_version
            )
        )
        return super().get_context_data(**context)


class ListBoxServicesView(OnlyStaffUserMixin, DjangoAdminContextMixin, TemplateView):
    title = 'List all FritzBox services'
    template_name = 'djfritz/list_box_services.html'

    def get_context_data(self, **context):
        fc = get_fritz_connection()
        context['services'] = fc.services

        if service_name := self.request.GET.get('service_name'):
            try:
                service = fc.services[service_name]
            except KeyError:
                messages.error(self.request, 'Invalid service name')
            else:
                context['current_service_name'] = service_name
                actions = service.actions
                context['service_actions'] = actions.keys()
                if action_name := self.request.GET.get('action_name'):
                    try:
                        action = service.actions[action_name]
                    except KeyError:
                        messages.error(self.request, 'Invalid service action name')
                    else:
                        context['current_action_name'] = action_name
                        arguments = []
                        for argument in action.arguments.values():
                            var = service.state_variables.get(argument.relatedStateVariable, '')
                            arguments.append(
                                {
                                    'argument_name': argument.name,
                                    'direction': argument.direction,
                                    'data_type': var.dataType,
                                }
                            )

                        context['action_arguments'] = arguments

        return super().get_context_data(**context)



from django.contrib.admin.utils import quote
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from djfritz.fritz_connection import FritzHostFilter
from djfritz.models.base import BaseTimetrackingTaggedModel


class HostModel(BaseTimetrackingTaggedModel):
    mac = models.CharField(
        # Stored in "RAW" format from FritzBox
        # uppercase with colons, e.g.: "AB:CD:EF:12:34:56"
        unique=True,
        db_index=True,
        blank=False,
        null=False,
        max_length=17,
        verbose_name=_('HostModel.mac.verbose_name'),
        help_text=_('HostModel.mac.help_text'),
    )
    ip_v4 = models.GenericIPAddressField(
        protocol='IPv4',
        blank=True,
        null=True,
        verbose_name=_('HostModel.ip_v4.verbose_name'),
        help_text=_('HostModel.ip_v4.help_text'),
    )
    ip_v6 = models.GenericIPAddressField(
        protocol='IPv6',
        blank=True,
        null=True,
        verbose_name=_('HostModel.ip_v6.verbose_name'),
        help_text=_('HostModel.ip_v6.help_text'),
    )
    name = models.CharField(
        blank=False,
        null=False,
        max_length=63,  # Max length of FritzBox host names ;)
        verbose_name=_('HostModel.name.verbose_name'),
        help_text=_('HostModel.name.help_text'),
    )
    last_status = models.BooleanField(
        # Is this host was active or not?
        null=False,
        verbose_name=_('HostModel.last_status.verbose_name'),
        help_text=_('HostModel.last_status.help_text'),
    )
    interface_type = models.CharField(
        # e.g.: "Ethernet",  "802.11", "HomePlug" or None
        blank=True,
        null=True,
        max_length=8,
        verbose_name=_('HostModel.interface_type.verbose_name'),
        help_text=_('HostModel.interface_type.help_text'),
    )
    address_source = models.CharField(
        # e.g.: "Static" or "DHCP"
        blank=False,
        null=False,
        max_length=6,
        verbose_name=_('HostModel.address_source.verbose_name'),
        help_text=_('HostModel.address_source.help_text'),
    )
    lease_time_remaining = models.PositiveIntegerField(
        blank=False,
        null=False,
        verbose_name=_('HostModel.lease_time_remaining.verbose_name'),
        help_text=_('HostModel.lease_time_remaining.help_text'),
    )

    WAN_ACCESS_STATE_CHOICES = [
        (FritzHostFilter.WAN_ACCESS_STATE_GRANTED, _('granted')),
        (FritzHostFilter.WAN_ACCESS_STATE_DENIED, _('denied')),
        (FritzHostFilter.WAN_ACCESS_STATE_ERROR, _('error')),
        (FritzHostFilter.WAN_ACCESS_STATE_UNKNOWN, _('unknown')),
    ]
    WAN_ACCESS_STATE_CHOICES_DICT = dict(WAN_ACCESS_STATE_CHOICES)

    wan_access = models.CharField(
        max_length=max(len(key) for key in WAN_ACCESS_STATE_CHOICES_DICT.keys()),
        choices=WAN_ACCESS_STATE_CHOICES,
        default=FritzHostFilter.WAN_ACCESS_STATE_UNKNOWN,
        editable=False,
        verbose_name=_('HostModel.wan_access.verbose_name'),
        help_text=_('HostModel.wan_access.help_text'),
    )

    def __str__(self):
        return f'Host "{self.name}" ({self.mac})'

    def get_change_url(self):
        opts = self._meta
        url = reverse(
            f'admin:{opts.app_label}_{opts.model_name}_change',
            args=(quote(self.pk),),
        )
        return url

    class Meta:
        verbose_name = _('HostModel.verbose_name')
        verbose_name_plural = _('HostModel.verbose_name_plural')



# Generated by Django 4.0.3 on 2022-03-31 17:38

from django.db import migrations, models
import tagulous.models.fields
import tagulous.models.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Tagulous_HostModel_tags',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField()),
                ('count', models.IntegerField(default=0, help_text='Internal counter of how many times this tag is in use')),
                ('protected', models.BooleanField(default=False, help_text='Will not be deleted when the count reaches 0')),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
                'unique_together': {('slug',)},
            },
            bases=(tagulous.models.models.BaseTagModel, models.Model),
        ),
        migrations.CreateModel(
            name='Tagulous_BaseTimetrackingTaggedModel_tags',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('slug', models.SlugField()),
                ('count', models.IntegerField(default=0, help_text='Internal counter of how many times this tag is in use')),
                ('protected', models.BooleanField(default=False, help_text='Will not be deleted when the count reaches 0')),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
                'unique_together': {('slug',)},
            },
            bases=(tagulous.models.models.BaseTagModel, models.Model),
        ),
        migrations.CreateModel(
            name='HostModel',
            fields=[
                ('create_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.create_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.create_dt.verbose_name')),
                ('update_dt', models.DateTimeField(blank=True, editable=False, help_text='ModelTimetrackingMixin.update_dt.help_text', null=True, verbose_name='ModelTimetrackingMixin.update_dt.verbose_name')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, help_text='BaseModel.id.help_text', primary_key=True, serialize=False, verbose_name='BaseModel.id.verbose_name')),
                ('mac', models.CharField(db_index=True, help_text='HostModel.mac.help_text', max_length=17, unique=True, verbose_name='HostModel.mac.verbose_name')),
                ('name', models.CharField(help_text='HostModel.name.help_text', max_length=63, verbose_name='HostModel.name.verbose_name')),
                ('last_status', models.BooleanField(help_text='HostModel.last_status.help_text', verbose_name='HostModel.last_status.verbose_name')),
                ('interface_type', models.CharField(blank=True, help_text='HostModel.interface_type.help_text', max_length=8, null=True, verbose_name='HostModel.interface_type.verbose_name')),
                ('address_source', models.CharField(help_text='HostModel.address_source.help_text', max_length=6, verbose_name='HostModel.address_source.verbose_name')),
                ('lease_time_remaining', models.PositiveIntegerField(help_text='HostModel.lease_time_remaining.help_text', verbose_name='HostModel.lease_time_remaining.verbose_name')),
                ('tags', tagulous.models.fields.TagField(_set_tag_meta=True, blank=True, case_sensitive=False, force_lowercase=False, help_text='BaseModel.tags.help_text', max_count=10, space_delimiter=False, to='djfritz.tagulous_hostmodel_tags', verbose_name='BaseModel.tags.verbose_name')),
            ],
            options={
                'verbose_name': 'HostModel.verbose_name',
                'verbose_name_plural': 'HostModel.verbose_name_plural',
            },
        ),
    ]



from django.contrib.auth.models import User
from model_bakery import baker

from djfritz.permissions import get_or_create_normal_user_group


def get_normal_pydjfritz_user(**baker_kwargs):
    pydjfritz_user_group = get_or_create_normal_user_group()[0]
    pydjfritz_user = baker.make(
        User,
        is_staff=True, is_active=True, is_superuser=False,
        **baker_kwargs
    )
    pydjfritz_user.groups.set([pydjfritz_user_group])
    return pydjfritz_user



from django.db.models.signals import post_migrate
from django.dispatch import receiver

from djfritz.permissions import get_or_create_normal_user_group, setup_normal_user_permissions


@receiver(post_migrate)
def post_migrate_callback(sender, **kwargs):
    normal_user_group, created = get_or_create_normal_user_group()
    if created:
        print(f'User group {normal_user_group} created')

    updated = setup_normal_user_permissions(normal_user_group)
    if updated:
        print(f'Update permissions for {normal_user_group}')



import os

from django.core.management import call_command
from django.core.management.commands.runserver import Command as RunServerCommand


class Command(RunServerCommand):
    help = "Run Django dev. Server"

    def verbose_call(self, command, **kwargs):
        self.stdout.write("\n")
        self.stdout.write("_" * 79)
        self.stdout.write(self.style.NOTICE(f" *** call '{command}' command with {kwargs}:"))
        self.stdout.write("\n")
        call_command(command, **kwargs)

    def handle(self, *args, **options):
        """
        INFO: The django reloader will call this multiple times!
        We check RUN_MAIN, that will be set in django.utils.autoreload
        So we can skip the first migrate run.
        """
        if os.environ.get("RUN_MAIN"):
            self.verbose_call("migrate", run_syncdb=True, interactive=False, verbosity=1)
            self.verbose_call("showmigrations", verbosity=1)

        self.verbose_call("runserver", **options)



# flake8: noqa: E405, F403

from djfritz_project.settings.base import *


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

SECRET_KEY = 'No individual secret... But this settings should only be used in tests ;)'

# Run the tests as on production: Without DBEUG:
DEBUG = False
TEMPLATE_DEBUG = False

ALLOWED_HOSTS = ('127.0.0.1', '0.0.0.0', 'localhost')

LOGGING['formatters']['colored']['format'] = (
    '%(log_color)s%(name)s %(levelname)8s %(cut_path)s:%(lineno)-3s %(message)s'
)



from bx_django_utils.humanize.pformat import pformat
from django.views.generic import TemplateView
from fritzconnection.lib.fritzhosts import FritzHosts

from djfritz.fritz_connection import FritzHostFilter, get_fritz_connection
from djfritz.views.base_views import DjangoAdminContextMixin, OnlyStaffUserMixin


class HostInformationView(OnlyStaffUserMixin, DjangoAdminContextMixin, TemplateView):
    title = 'Get information about registered hosts'
    template_name = 'djfritz/host_information.html'

    def get_context_data(self, **context):
        fc = get_fritz_connection()

        fh = FritzHosts(fc=fc)
        context['hosts'] = sorted(fh.get_hosts_info(), key=lambda x: (not x['status'], x['ip']))
        if ip := self.request.GET.get('ip'):
            context['current_ip'] = ip

            fhf = FritzHostFilter(fc=fc)

            if wan_access := self.request.GET.get('wan_access'):
                # Change Internet access for one IP address
                if wan_access == 'disallow':
                    disallow_wan_access = 1
                elif wan_access == 'allow':
                    disallow_wan_access = 0
                else:
                    raise TypeError('Unknown value')

                fhf.disallow_wan_access(ip=ip, state=disallow_wan_access)

            context['wan_access_state'] = fhf.wan_access_state(ip=ip)

        return super().get_context_data(**context)


class MeshTopologyView(OnlyStaffUserMixin, DjangoAdminContextMixin, TemplateView):
    title = 'Get raw mesh topology'
    template_name = 'djfritz/mesh_topology.html'

    def get_context_data(self, **context):
        fc = get_fritz_connection()

        fh = FritzHosts(fc=fc)
        mesh_topology = fh.get_mesh_topology()

        context['mesh_topology_pformat'] = pformat(mesh_topology)

        return super().get_context_data(**context)



import logging
import time

from django.utils import timezone
from fritzconnection import FritzConnection
from fritzconnection.core.exceptions import FritzActionFailedError, FritzConnectionException
from fritzconnection.lib.fritzbase import AbstractLibraryBase


logger = logging.getLogger(__name__)


class LazyFritzConnection:
    fc = None
    last_connection = None

    def __call__(self) -> FritzConnection:
        if self.fc is None:
            try:
                self.fc = FritzConnection()
            except FritzConnectionException as err:
                logger.error('Can not connect to FritzBox: %s', err)
            else:
                self.last_connection = timezone.now()
        return self.fc


get_fritz_connection = LazyFritzConnection()


class FritzHostFilter(AbstractLibraryBase):
    SERVICE = 'X_AVM-DE_HostFilter1'

    def _action(self, actionname, *, arguments=None, **kwargs):
        return self.fc.call_action(self.SERVICE, actionname, arguments=arguments, **kwargs)

    WAN_ACCESS_STATE_GRANTED = 'granted'
    WAN_ACCESS_STATE_DENIED = 'denied'
    WAN_ACCESS_STATE_ERROR = 'error'
    KNOWN_WAN_ACCESS_STATES = (
        # These states will the FritzBox return. If not -> use "unknown" below
        WAN_ACCESS_STATE_GRANTED,
        WAN_ACCESS_STATE_DENIED,
        WAN_ACCESS_STATE_ERROR,
    )
    WAN_ACCESS_STATE_UNKNOWN = 'unknown'

    def get_wan_access_state(self, ip):
        """
        Returns the state of WANAccess for the given LAN device’s IP address.
        States are:
            "granted" The LAN device has access to WAN.
            "denied" The LAN device has no access to WAN.
            "error" Something went wrong, the state could not yet be retrieved.

        Needs authenticated login to FritzBox.
        """
        raw_state = self._action('GetWANAccessByIP', NewIPv4Address=ip)
        state = raw_state['NewWANAccess']
        logger.info('GetWANAccessByIP: ip=%r has state=%r (Raw: %r)', ip, state, raw_state)
        if state not in self.KNOWN_WAN_ACCESS_STATES:
            return self.WAN_ACCESS_STATE_UNKNOWN
        return state

    def set_wan_access_state(self, ip: str, allow: bool) -> str:
        """
        Change the internet access for given device’s IP address.

            allow is True -> WAN access granted
            allow is False -> WAN access denied

        (Needs authenticated login to FritzBox.)
        """
        state_map = {
            True: (0, self.WAN_ACCESS_STATE_GRANTED),
            False: (1, self.WAN_ACCESS_STATE_DENIED),
        }
        new_disallow, expected_state = state_map[allow]

        logger.info('DisallowWANAccessByIP: Set disallow=%r for ip=%r', new_disallow, ip)
        self._action('DisallowWANAccessByIP', NewIPv4Address=ip, NewDisallow=new_disallow)

        state = None
        for sec in range(30, 1, -1):
            state = self.get_wan_access_state(ip=ip)
            if state != expected_state:
                logger.info(f'State {state} is not {expected_state}... (max wait {sec}sec.)')
                time.sleep(1)
            else:
                return state

        raise FritzActionFailedError(
            f'Setting WAN access for {ip} to {expected_state} failed, new state is: {state}'
        )

    def allow_wan_access(self, ip: str) -> str:
        """
        Set WAN access state to "granted" (Needs to be authenticated)
        """
        return self.set_wan_access_state(ip=ip, allow=True)

    def disallow_wan_access(self, ip: str) -> str:
        """
        Set WAN access state to "denied" (Needs to be authenticated)
        """
        return self.set_wan_access_state(ip=ip, allow=False)



from django.test import TestCase


class AdminAnonymousTests(TestCase):
    """
    Anonymous will be redirected to the login page.
    """

    def test_login_en(self):
        response = self.client.get('/admin/', secure=True, HTTP_ACCEPT_LANGUAGE='en')
        self.assertRedirects(
            response, expected_url='/admin/login/?next=/admin/', fetch_redirect_response=False
        )

    def test_login_de(self):
        response = self.client.get('/admin/', secure=True, HTTP_ACCEPT_LANGUAGE='de')
        self.assertRedirects(
            response, expected_url='/admin/login/?next=/admin/', fetch_redirect_response=False
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['djfritz',
 'djfritz.admin',
 'djfritz.management',
 'djfritz.management.commands',
 'djfritz.migrations',
 'djfritz.models',
 'djfritz.services',
 'djfritz.templatetags',
 'djfritz.tests',
 'djfritz.tests.fixtures',
 'djfritz.views',
 'djfritz_project',
 'djfritz_project.settings',
 'djfritz_project.tests']

package_data = \
{'': ['*'],
 'djfritz': ['locale/de/LC_MESSAGES/*',
             'locale/en/LC_MESSAGES/*',
             'templates/admin/djfritz/hostmodel/*',
             'templates/djfritz/*',
             'templates/djfritz/includes/*'],
 'djfritz_project': ['templates/admin/*']}

install_requires = \
['bx_django_utils',
 'bx_py_utils',
 'colorlog',
 'django',
 'django-admin-sortable2',
 'django-debug-toolbar',
 'django-reversion-compare',
 'django-tagulous',
 'django-tools',
 'fritzconnection']

entry_points = \
{'console_scripts': ['devshell = djfritz_project.dev_shell:devshell_cmdloop',
                     'run_testserver = '
                     'djfritz_project.manage:start_test_server']}

setup_kwargs = {
    'name': 'django-fritzconnection',
    'version': '0.0.2rc0',
    'description': 'Web based FritzBox management using Python/Django.',
    'long_description': '# django-fritzconnection\n\n![django-fritzconnection @ PyPi](https://img.shields.io/pypi/v/django-fritzconnection?label=django-fritzconnection%20%40%20PyPi)\n![Python Versions](https://img.shields.io/pypi/pyversions/django-fritzconnection)\n![License GPL V3+](https://img.shields.io/pypi/l/django-fritzconnection)\n\nWeb based FritzBox management using Python/Django.\n\nCurrent state: **early development stage**\n\nFeatures:\n\n* actions:\n  * Change WAN access of a host\n* models:\n  * HostModel\n    * "Static" storage for all `FritzHosts().get_hosts_info()` information\n    * Update in Admin via change list tools link and manage command\n* a few "test" views:\n  * Host information\n    * Get information about registered hosts\n    * Get raw mesh topology\n  * Diagnose\n    * Test FritzBox connection\n    * List all FritzBox services\n\n\n[![Install django-fritzconnection with YunoHost](https://install-app.yunohost.org/install-with-yunohost.svg)](https://install-app.yunohost.org/?app=django-fritzconnection)\n\n> *This package allows you to install django-fritzconnection quickly and simply on a YunoHost server.\nIf you don\'t have YunoHost, please consult [the guide](https://yunohost.org/#/install) to learn how to install it.*\n\nPull requests welcome ;)\n\n\n## Quick start for developers\n\n```\n~$ git clone https://github.com/jedie/django-fritzconnection.git\n~$ cd django-fritzconnection\n~/django-fritzconnection$ ./devshell.py\n...\nDeveloper shell - djfritz - v0.0.2.rc0\n...\n\n(djfritz) run_testserver\n```\n\n## versions\n\n* [*dev*](https://github.com/jedie/django-fritzconnection/compare/v0.0.2.rc0...main)\n  * TBC\n* [v0.0.2.rc0 - 02.04.2022](https://github.com/jedie/django-fritzconnection/compare/v0.0.1-alpha...v0.0.2.rc0)\n  * Store Host information\n  * Possible to set WAN access for one host\n* v0.0.1-alpha - 24.03.2022\n  * init the project\n',
    'author': 'JensDiemer',
    'author_email': 'git@jensdiemer.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jedie/django-fritzconnection',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0.0',
}


setup(**setup_kwargs)



from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType


NORMAL_USER_GROUP_NAME = 'normal user'


def get_permissions(*models):
    content_types = []
    for model in models:
        content_types.append(ContentType.objects.get_for_model(model))

    return Permission.objects.filter(content_type__in=content_types)


def get_or_create_normal_user_group():
    """
    Will be called by:
        djfritz.signals.post_migrate_callback()
    """
    return Group.objects.get_or_create(name=NORMAL_USER_GROUP_NAME)


def setup_normal_user_permissions(normal_user_group):
    """
    Setup django-fritzconnection "normal user" permissions.
    Will be called by:
        djfritz.signals.post_migrate_callback()
    """
    assert normal_user_group.name == NORMAL_USER_GROUP_NAME
    permissions = get_permissions(
        # TODO: Add model classes here
    )
    existing_permissions = normal_user_group.permissions.all()

    if set(permissions) != set(existing_permissions):
        normal_user_group.permissions.set(permissions)
        return True

    return False



import uuid

import tagulous.models
from bx_django_utils.models.timetracking import TimetrackingBaseModel
from django.db import models
from django.utils.translation import gettext_lazy as _


class BaseTimetrackingTaggedModel(TimetrackingBaseModel):
    id = models.UUIDField(
        primary_key=True,
        default=uuid.uuid4,
        editable=False,
        verbose_name=_('BaseModel.id.verbose_name'),
        help_text=_('BaseModel.id.help_text'),
    )
    tags = tagulous.models.TagField(
        blank=True,
        case_sensitive=False,
        force_lowercase=False,
        space_delimiter=False,
        max_count=10,
        verbose_name=_('BaseModel.tags.verbose_name'),
        help_text=_('BaseModel.tags.help_text'),
    )

    def __str__(self):
        return self.name

    class Meta:
        abstract = True



import subprocess
import sys
from pathlib import Path
from unittest import TestCase

from dev_shell.utils.assertion import assert_is_file

import djfritz


BASE_PATH = Path(djfritz.__file__).parent.parent


def call_run_testserver(*args):
    dev_shell_py = BASE_PATH / 'devshell.py'
    assert_is_file(dev_shell_py)
    output = subprocess.check_output(
        [sys.executable, str(dev_shell_py), 'run_testserver'] + list(args),
        stderr=subprocess.STDOUT,
        text=True
    )
    return output


class RunTestServerTestCase(TestCase):
    def test_run_testserver(self):
        output = call_run_testserver('--help')
        assert 'usage: manage.py run_testserver' in output
        assert 'Run Django dev. Server' in output
        assert 'Optional port number, or ipaddr:port' in output

    def test_pass_wrong_addrport(self):
        output = call_run_testserver('not-ip:no-port')
        assert "call 'runserver' command with" in output
        assert (
            'CommandError: "not-ip:no-port" is not a valid port number or address:port pair.'
        ) in output
